package mandelbrot;

import javax.swing.SwingUtilities;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.ExampleMode;
import org.kohsuke.args4j.Option;

/**
 * Class to run the application. Call java mandelbrot.Start [options] to run the
 * application.
 * 
 * @author ahoefer
 * 
 */
public final class Start {

	/**
	 * args4j Option. If set and more than one processor is available the
	 * Mandelbrot will be computed in parallel. Default is sequential.
	 */
	@Option(name = "-p", aliases = { "--parallel" }, usage = "If set and more than one processor is available the Mandelbrot will be computed in parallel. Default is sequential.")
	public static boolean parallel = false;

	/**
	 * args4j option. If set the program will terminate after computing the
	 * Mandelbrot instead of displaying it in a window. This is useful for time
	 * measurement.
	 */
	@Option(name = "-w", aliases = { "--supresswindow" }, usage = "If set the program will terminate after computing the Mandelbrot instead of displaying it in a window. This is useful for time measurement.")
	public static boolean supressWindow = false;

	/**
	 * args4j option. The side length of the Mandelbrot square in pixels.
	 * Default is 600.
	 */
	@Option(name = "-s", aliases = { "--size" }, usage = "The side length of the Mandelbrot square in pixels. Default is 600.")
	public static int size = 600;

	/**
	 * args4j Option. The number of threads to use for the parallel computation.
	 * Defaults to the number of processors of the machine the program is
	 * running on.
	 */
	@Option(name = "-t", aliases = { "--threads" }, usage = "The number of threads to use for the parallel computation. Defaults to the number of processors of the machine the program is running on.")
	public static int threads = Runtime.getRuntime().availableProcessors();

	/**
	 * args4j option. Set an alternative color map. Allowed values are:
	 * <ul>
	 * <li>mandelbrot.GrayColor</li>
	 * <li>mandelbrot.RGBColorMap (Default)</li>
	 * </ul>
	 */
	@Option(name = "-c", aliases = { "--colormap" }, usage = "Set an alternative color map. Allowed values are:\nmandelbrot.GrayColorMap\nmandelbrot.RGBColorMap (Default)")
	public static String cmClassName = "mandelbrot.RGBColorMap";

	private Start() {
	}

	/**
	 * The entry point for the application.
	 * 
	 * @param args
	 *            See the description of the args4j options for valid arguments.
	 */
	public static void main(String[] args) {
		new Start().parseArgs(args);
		final AbstractMandelbrot mandelbrot = createMandelbrot();

		if (supressWindow) {
			mandelbrot.computeMandelbrot();
			return;
		}

		final ColorMap colorMap = createColorMap();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new MainFrame(mandelbrot, colorMap).setVisible(true);
			}
		});
	}

	private static AbstractMandelbrot createMandelbrot() {
		int s = Math.abs(size);
		return new SequentialMandelbrot(s, s);
	}

	/**
	 * Creates the color map to use for the Mandelbrot depending on the
	 * <code>cmClassName</code> args4j Option.
	 * 
	 * @return the color map
	 */
	private static ColorMap createColorMap() {
		if (cmClassName.equals("mandelbrot.GrayColorMap")) {
			return new GrayColorMap();
		}
		return new RGBColorMap();
	}

	/**
	 * Parses the command line arguments using <a
	 * href="https://args4j.dev.java.net/">args4j</a> .
	 * 
	 */
	private void parseArgs(String[] args) {
		CmdLineParser parser = new CmdLineParser(this);
		parser.setUsageWidth(80);

		try {
			parser.parseArgument(args);
		} catch (CmdLineException e) {
			System.err.println(e.getMessage());
			System.err.println("java -jar mandelbrot.jar [options...]");
			parser.printUsage(System.err);
			System.err.println();
			System.err.println("Example: java -jar mandelbrot.jar"
					+ parser.printExample(ExampleMode.ALL));
		}
	}
}
